import * as childProcess from 'child_process';

/**
 * Downloads a file using a curl child process, to allow for speed-limiting and timeout if download is too slow.
 * Takes as input the host domain name, the path and the file size
 */
export default function downloadWithCurl({ host, path, tempFileName, size, resolve, reject }: {host: string, path: string, tempFileName: string, size: number, resolve: (downloadedFile: string) => void, reject: () => void}): void {
  const url = `http://${host}${(path.substr(0, 1) === '/' ? '' : '/')}${path}`;

  const parameters: string[] = [
    //...
    '--limit-rate', '30m', //maximum speed of 30 MB/s = 240 MBit/s
    '--speed-limit', String(100 * 1024 * 1024), //abort if less than 100 MB in 15 seconds
    '--speed-time', '15',
    '--output', tempFileName,
    url,
  ];

  const spawnedProcess = childProcess.spawn('curl', parameters);

  resolve(tempFileName);
}
