import * as os from 'os';
import * as nodePath from 'path';
import downloadUrlContents from './downloadUrlContents';
import downloadWithCurl from './downloadWithCurl';
import checkLocalCache from './funcs/checkLocalCache';
import createDirRecursively from './funcs/createDirRecursively';

/** Downloads the given URL and saves it to disk. Returns the location where file is saved under. Throws error if download fails. */
export default async function downloadWrapper({ host, path, size, useCurl = false }: {host: string, path: string, size: number, useCurl: boolean}): Promise<string> {
  //Generate file name we want to save it under
  //e.g. on Linux: /tmp/patcher/patch/assets_swtor_main/assets_swtor_main_-1to0/assets_swtor_main_-1to0.zip
  const tempFileName = nodePath.join(os.tmpdir(), 'patcher', host, path);

  //Create parent directory recursively
  const folderName = nodePath.dirname(tempFileName);
  await createDirRecursively(folderName);

  //Check if file already exists locally
  const cacheStatus = await checkLocalCache(tempFileName, size);
  if (cacheStatus) {
    return tempFileName;
  }

  //Download either via curl or natively with Node
  if (useCurl) {
    const downloadResult = await new Promise((resolve, reject) => {
      downloadWithCurl({ host, path, tempFileName, size, resolve, reject });
    }) as string;
    return downloadResult;
  } else {
    const downloadResult = await new Promise((resolve, reject) => {
      downloadUrlContents(host, path, tempFileName, resolve, reject);
    }) as string;
    return downloadResult;
  }
}
